//  Copyright (c) 1999-2008  Pavel Rychly

#include "corpus.hh"

using namespace std;

Structure::Structure (CorpInfo *i, const string &path, const string &n)
    : Corpus (i, Struct_type), 
      rng (create_ranges(path + ".rng", i->opts["TYPE"])), name (n), 
    endtagstring ("</" + name + '>')
{
}


class StructPosAttr : public PosAttr
{
    Structure *st;
    PosAttr *pa;
    Position lastrng;
    Position locate_rng (Position pos) {
        if (st->rng->beg_at (lastrng) <= pos && st->rng->end_at (lastrng) >= pos)
            return lastrng;
        else {
            Position n = st->rng->num_at_pos (pos);
            if (n >= 0)
                lastrng = n;
            return n;
        }
    }
public:
    StructPosAttr (Structure *st, PosAttr *pa)
        : PosAttr (pa->attr_path, (st->name + '.' + pa->name), pa->locale),
          st (st), pa(pa), lastrng (0)
    {}
    virtual ~StructPosAttr () {}
    virtual int id_range () {return pa->id_range();}
    virtual const char* id2str (int id) {return pa->id2str (id);}
    virtual int str2id(const char *str) {return pa->str2id (str);}
    virtual int pos2id (Position pos) {
        Position n = locate_rng (pos);
        return (n == -1) ? -1 : pa->pos2id (n);
    }
    virtual const char* pos2str (Position pos) {
        Position n = locate_rng (pos);
        return (n == -1) ? "" : pa->pos2str (n);
    }
    virtual IDIterator *posat (Position pos) {return NULL; /*XXX*/}
    virtual TextIterator *textat (Position pos) {return NULL; /*XXX*/}
    virtual FastStream *id2poss (int id) {return NULL; /*XXX*/}
    virtual FastStream *regexp2poss (const char *pat, bool) {return NULL; /*XXX*/}
    virtual FastStream *compare2poss (const char *pat, int, bool) {return NULL; /*XXX*/}
    virtual Generator<int> *regexp2ids (const char *pat, bool ignorecase) 
        {return pa->regexp2ids (pat,ignorecase);}
    virtual NumOfPos freq (int id) {return pa->freq (id);}
    virtual NumOfPos norm (int id) {return pa->norm (id);}
    virtual NumOfPos size () {return 0;/*XXX*/}
};



PosAttr* Corpus::get_struct_pos_attr (const std::string &strname, 
                                      const std::string &attname)
{
    Structure* s = get_struct (strname);
    PosAttr* a = s->get_attr (attname);
    return new StructPosAttr (s, a); // XXX mem leak
}


/*
XXX nesting musi byt ulozeno v .rng a tady se musi predavat
    pro nesting > 0 tam musi byt i final (max end)
RangeStream *Structure::whole()
{
    return new whole_range (&rng);
}


RangeStream *Structure::part(FastStream *filter)
{
    return new part_range (&rng, filter);
}

*/

// vim: ts=4 sw=4 sta et sts=4 si cindent tw=80:

